<?php
/**
 * @package CCL
 * @version 0.0.1
 * @author trustmaster
 * @copyright (c) 2009 Vladimir Sibirov
 * @license BSD
 */

/**
 * Enumeration (ENUM) parameter class
 */
class Enum extends Param {
	/**
	 * @var array Possible alternatives
	 */
	public $values = array();
	/**
	 * @var array Captions for values
	 */
	public $captions = array();

	/**
	 * Parameter constructor
	 *
	 * @param int $type Value type
	 * @param string $name Parameter name
	 * @param array $values Value alternatives
	 * @param string $default Default value
	 * @param bool $display Render it in templates
	 * @param bool $required Require it from user input
	 */
	public function __construct($type, $name, $values, $default = NULL, $display = TRUE, $required = FALSE)
	{
		if (!is_array($values) || count($values) == 0)
		{
			throw new Exception('Enum param must contain an array of alternatives');
			// TODO better exception handling
		}
		$this->values = $values;
		parent::__construct($type, $name, $display, $required);
		if (!is_null($default))
		{
			$this->default = $default;
		}
		$this->captions = $this->get_captions();
	}

	/**
	 * Gets an array of acceptable alternative values for this enumeration
	 * @return array Possible values
	 */
	public function get_alternatives()
	{
		return $this->values;
	}

	/**
	 * Gets an array of captions for alternatives from lang files if possible
	 * @return array Captions for values
	 */
	private function get_captions()
	{
		global $L;
		foreach ($this->values as $val)
		{
			// TODO $L['param_' . $col_prefix . $this->name . '_' . $val]
			if (isset($L['param_' . $this->name . '_' . $val]))
			{
				$captions[] = $L['param_' . $this->name . '_' . $val];
			}
			elseif (isset($L[$this->name . '_' . $val]))
			{
				$captions[] = $L[$this->name . '_' . $val];
			}
			elseif (isset($L[$val]))
			{
				$captions[] = $L[$val];
			}
			else
			{
				$captions[] = $val;
			}
		}
		return $captions;
	}

	/**
	 * Imports parameter value from user input
	 * @param string $method Request method: P - post, G - get.
	 * @param string $prefix Input name prefix
	 * @return mixed Value on success or NULL if validation failed
	 */
	public function import($method = 'P', $prefix = '')
	{
		// TODO throw exceptions
		$value = sed_import($prefix . $this->name, $method, 'TXT');
		if (in_array($value, $this->values))
		{
			return $value;
		}
		else
		{
			if (!$this->required && !is_null($this->default))
			{
				return $this->default;
			}
			elseif (!$this->required)
			{
				return '';
			}
			else
			{
				return NULL;
			}
		}
	}

	/**
	 * Renders param value display in a correct way
	 * @param mixed $value
	 * @return string Rendered display element
	 */
	public function display($value)
	{
		$index = array_search($value, $this->values);
		if ($index === FALSE)
		{
			return sed_cc($value);
		}
		else
		{
			return sed_cc($this->captions[$index]);
		}
	}

	/**
	 * Renders param input display in a correct way
	 * @param mixed $value
	 * @param string $prefix Input name prefix
	 * @return string
	 * @see Param::title, Param::hint
	 */
	public function display_input($value = NULL, $prefix = '')
	{
		if (is_null($value) && !is_null($this->default))
		{
			$value = $this->default;
		}
		$class = empty($this->input_class) ? '' : ' class="' . $this->input_class . '"';
		$disp = '<select name="' . $prefix . $this->name . '"' . $class . '>';
		if (!$this->required) $disp .= '<option value="">--</option>';
		$cnt = count($this->values);
		for($i = 0; $i < $cnt; $i++)
		{
			$selected = (is_null($value) || $value != $this->values[$i]) ? '' : ' selected="selected"';
			$disp .= '<option value="' . sed_cc($this->values[$i]) . '"' . $selected . '>'
				. sed_cc($this->captions[$i]) . '</option>';
		}
		$disp .= '</select>';
		return $disp;
	}
}
?>
