<?php
/**
 * @package CCL
 * @version 0.0.1
 * @author trustmaster
 * @copyright (c) 2009 Vladimir Sibirov
 * @license BSD
 */

/**
 * Controller item class. Controller Units operate on Items
 *
 * @param int $id Primary key
 */
class Item extends Instance
{
	/**
	 * Instance constructor. Creates object from SQL result row, loads by ID or just leaves
	 * blank to be filled later.
	 * @param Unit $unit Reference to parent unit
	 * @param mixed $source ID or query result row
	 */
	public function __construct($unit = NULL, $source = NULL)
	{
		if ($unit instanceof Unit)
		{
			$this->entity = $unit;
		}
		else
		{
			$source = $unit;
		}
		
		if (is_int($source))
		{
			// Load from db by id
			$source = $this->get_by_id($source);
		}

		parent::__construct($this->entity, $source);
	}

	/**
	 * Fetches a row from the database by ID
	 * @param int $id Instance ID
	 * @return array Query result row or NULL if no items were found
	 */
	private function get_by_id($id)
	{
		$res = sed_sql_query('SELECT * FROM `' . $this->entity->table_name . '`
			WHERE `' . $this->entity->col_prefix . 'id` = ' . (int) $id );
		if (sed_sql_numrows($res) == 1)
		{
			$row = sed_sql_fetchassoc($res);
		}
		else
		{
			$row = NULL;
		}
		sed_sql_freeresult($res);
		return $row;
	}

	/**
	 * Loads object from database
	 * @param int $id Instance ID
	 * @return bool TRUE on success, FALSE on error
	 */
	public function load($id)
	{
		if (is_int($id))
		{
			return $this->assign_from_row($this->get_by_id($id));
		}
		else
		{
			return FALSE;
		}
	}

	/**
	 * Imports data from user input, validates it and stores in the object
	 *
	 * @param string $method Input method 'P' for POST or 'G' for GET
	 * @param string $prefix Input name prefix, defaults to database column prefix
	 * @return bool TRUE on successfull validation or FALSE if validation has failed
	 * @see Unit::display_add()
	 */
	public function import($method = 'P', $prefix = NULL)
	{
		// TODO throw exceptions
		$valid = TRUE;
		if (is_null($prefix))
		{
			$prefix = $this->entity->col_prefix;
		}
		foreach ($this->entity as $name => $param)
		{
			if (!$param->primary_key || empty($this->{$name}))
			{
				$value = $param->import($method, $prefix);
				if (is_null($value))
				{
					// TODO catch exceptions
					$valid = FALSE;
				}
				else
				{
					$this->data[$name] = $value;
				}
			}
		}
		return $valid;
	}

	/**
	 * Gets a referenced object as result row to be passed to exact class constructor
	 * @param string $param_name Reference parameter name
	 * @return Item An item that is referenced to
	 */
	public function get_referenced_item($param_name)
	{
		if (empty($this->data[$param_name]))
		{
			return NULL;
		}
		else
		{
			$class = $this->entity[$param_name]->entity->item_class;
			return new $class($this->entity[$param_name]->entity, (int) $this->data[$param_name]);
		}
	}

	/**
	 * Gets all items which reference to this item with a specific entity parameter.
	 * @param Unit $unit Unit that references to this unit
	 * @param string $param_name Referencing parameter name
	 * @return array List of Items
	 */
	public function get_referencing_items($unit, $param_name = '')
	{
		$items = array();
		$class = $unit->item_class;
		if (empty($param_name))
		{
			foreach ($unit as $name => $param)
			{
				if ($param instanceof Reference && $param->entity == $this->entity)
				{
					$param_name = $name;
					break;
				}
			}
			if (empty($param_name)) return NULL;
		}
		$res = sed_sql_query('SELECT * FROM `' . $unit->table_name . '`
			WHERE `' . $unit->col_prefix . $param_name . '` = ' . $this->id);
		while ($row = sed_sql_fetchassoc($res))
		{
			$items[] = new $class($unit, $row);
		}
		sed_sql_freeresult($res);
		return $items;
	}

	/**
	 * Gets items from another Unit which reference to this Item with a many-to-many Relation
	 * @param Unit $unit Target unit which relates to this unit
	 * @param Relation $relation Relation which connects units
	 * @param string $relation_source Relation parameter name which references to this item
	 * @param string $relation_target Relation parameter which references to target unit
	 * @return array List of Items
	 */
	public function get_related_items($unit, $relation, $relation_source = '', $relation_target = '')
	{
		$items = array();
		$class = $unit->item_class;
		if (empty($relation_source) || empty($relation_target))
		{
			foreach ($relation as $name => $param)
			{
				if ($param instanceof Reference)
				{
					if (empty($relation_source) && $param->entity == $this->entity)
					{
						$relation_source = $name;
					}
					elseif (empty($relation_target) && $param->entity == $unit)
					{
						$relation_target = $name;
					}
				}
			}
		}
		$rels = $this->get_referencing_items($relation, $relation_source);
		foreach ($rels as $rel)
		{
			$items[] = $rel->get_referenced_item($relation_target);
		}
		return $items;
	}

	/**
	 * Single object display using XTemplate object.
	 * Controller logic for XTemplate: assings tags, parses blocks.
	 * TPL designer must specify unit parameters explicitly.
	 * TPL example:
	 * <!-- BEGIN: MY_UNIT -->
	 * <stong>{FOO_TITLE}:</strong> {FOO}
	 * <stong>{BAR_TITLE}:</strong> {BAR}
	 * <!-- END: MY_UNIT -->
	 *
	 * @param XTemplate $tpl Layout template object
	 * @param string $block Full block name to render the object in, e.g. 'MAIN.MY_UNIT'
	 */
	public function display($tpl, $block)
	{
		foreach ($this->entity as $name => $param)
		{
			$tpl->assign(array(
				mb_strtoupper($name) . '_TITLE' => sed_cc($param->title),
				mb_strtoupper($name) => $param->display($this->{$name})
			));
		}
		// TODO display related items/links
		// TODO check edit permissions
		if (defined('SED_MODULE'))
		{
			$tpl->assign(array(
				'DISPLAY_URL' => sed_url(SED_MODULE, 'z=' . $this->entity->zone . '&m=display&id=' . $this->id),
				'EDIT_URL' => sed_url(SED_MODULE, 'z=' . $this->entity->zone . '&m=edit&id=' . $this->id),
				'REMOVE_URL' => sed_url(SED_MODULE, 'z=' . $this->entity->zone . '&m=remove&id=' . $this->id)
			));
		}
		else
		{
			global $pl;
			$tpl->assign(array(
				'DISPLAY_URL' => sed_url('plug', 'e=' . $pl['pl_code'].'&z=' . $this->entity->zone . '&m=display&id=' . $this->id),
				'EDIT_URL' => sed_url('plug', 'e=' . $pl['pl_code'].'&z=' . $this->entity->zone . '&m=edit&id=' . $this->id),
				'REMOVE_URL' => sed_url('plug', 'e=' . $pl['pl_code'].'&z=' . $this->entity->zone . '&m=remove&id=' . $this->id)
			));
		}
		$tpl->parse($block);
	}

	/**
	 * Row-based automatic unit render
	 * TPL designer specifies parameters implicitly with row block.
	 * TPL example:
	 * <!-- BEGIN: MY_UNIT -->
	 * <!-- BEGIN: MY_PARAM -->
	 * <stong>{PARAM_TITLE}:</strong> {PARAM_VALUE}
	 * <!-- END: MY_PARAM -->
	 * <!-- END: MY_UNIT -->
	 *
	 * @param XTemplate $tpl Layout template object
	 * @param string $block Full block name to render the object in, e.g. 'MAIN.MY_UNIT'
	 * @param string $row_block Row block name for parameter row, e.g. 'MY_PARAM'
	 * @param string $title_tag Tag name for parameter title
	 * @param string $value_tag Tag name for parameter value
	 */
	public function display_auto($tpl, $block, $row_block = 'PARAM', $title_tag = 'PARAM_TITLE', $value_tag = 'PARAM_VALUE')
	{
		global $L, $pl;
		foreach ($this->entity as $name => $param)
		{
			$tpl->assign(array(
				$title_tag => sed_cc($param->title),
				$value_tag => $param->display($this->{$name})
			));
			$tpl->parse($block . '.' . $row_block);
		}
		foreach ($this->entity->references as $ref)
		{
			foreach ($ref as $name => $param)
			{
				if ($param instanceof Reference && $param->entity == $this->entity)
				{
					$title = empty($L[$ref->name]) ? $ref->name : $L[$ref->name];
					$z = mb_strtolower($ref->name); // FIXME i18n
					$show_url = defined('SED_MODULE') ? sed_url(SED_MODULE, "z=$z&$name=" . $this->id) : sed_url('plug', "e={$pl['pl_code']}&z=$z&$name=" . $this->id);
					$tpl->assign(array(
						$title_tag => sed_cc($title),
						$value_tag =>
							'<a href="'.$show_url.'">'.$L['Show'].'</a>'
					));
					$tpl->parse($block . '.' . $row_block);
				}
			}
		}
		$tpl->parse($block);
	}
}
?>
