<?php
/**
 * @package CCL
 * @version 0.0.1
 * @author trustmaster
 * @copyright (c) 2009 Vladimir Sibirov
 * @license BSD
 */

/**
 * Enumeration with available parameter types. This is related to Controller
 * only, the exact data type is set in Model.
 *
 * @see Param
 */
abstract class Param_type {
	/**
	 * Boolean parameter
	 */
	const bool = 0;
	/**
	 * Integer parameter
	 */
	const int = 1;
	/**
	 * Real number with floating point
	 */
	const float = 2;
	/**
	 * Text/string/byte stream
	 */
	const string = 3;
	/**
	 * Maximal parameter type code provided by this class
	 */
	const max_code = 3;
}

/**
 * Object parameter
 * 
 * @see Param_type
 */
class Param {
	/**
	 * @var int Parameter type
	 */
	public $type;
	/**
	 * @var string Parameter name
	 */
	public $name;
	/**
	 * @var bool This parameter is primary key of the entity
	 */
	public $primary_key = FALSE;
	/**
	 * @var bool Display this parameter in forms, otherwise it is hidden
	 */
	public $display = TRUE;
	/**
	 * @var string Parameter title
	 */
	public $title;
	/**
	 * A user must specify this parameter non-empty
	 * @var bool Parameter is required
	 */
	public $required = FALSE;
	/**
	 * Optional default value
	 * @var mixed Default value
	 */
	public $default = NULL;
	/**
	 * Used on display in forms
	 * @var string Parameter hint
	 */
	public $hint = '';
	/**
	 * For string parameters only
	 * @var bool Allow HTML and special characters
	 */
	public $allow_html = FALSE;
	/**
	 * Optional class attribute for XHTML input
	 * @var string Class or classes
	 */
	public $input_class = NULL;

	/**
	 * Constructs parameter object
	 *
	 * @global array $L Cotonti locale strings
	 * @param int $type Param_type
	 * @param string $name Parameter name
	 * @param bool $display Render it in templates
	 * @param bool $required Require it from user input
	 * @param bool $primary_key Is primary key of the entity
	 */
	public function __construct($type, $name, $display = TRUE,
		 $required = FALSE, $primary_key = FALSE)
	{
		global $L;
		if ($type < 0 || $type > Param_type::max_code)
		{
			throw new Exception('Unknown parameter type');
		}
		$this->type = $type;
		if (empty($name))
		{
			throw new Exception('Parameter name is empty');
		}
		$this->name = $name;
		$this->primary_key = $primary_key;
		$this->display = $display && !$primary_key;
		$this->required = $required;

		// TODO param_$col_prefix_$name
		if (!empty($L["param_$name"]))
		{
			$this->title = $L["param_$name"];
		}
		elseif (!empty($L[$name]))
		{
			$this->title = $L[$name];
		}
		else
		{
			$this->title = $name;
		}

		$this->hint = empty($L["hint_$name"]) ? '' : $L["hint_$name"];
	}

	/**
	 * Casts a value due to parameter type
	 * @param mixed $value Raw value
	 * @return mixed Casted value
	 */
	public function cast($value)
	{
		switch ($this->type)
		{
			case Param_type::bool:
				return $value = (bool) $value;
				break;
			case Param_type::float:
				return (float) $value;
				break;
			case Param_type::int:
				return (int) $value;
				break;
			default:
				return $value;
				break;
		}
	}

	/**
	 * Prepares a parameter value for use in database statement
	 * @param mixed $value Source value
	 * @return mixed Prepared value
	 */
	public function prepare($value)
	{
		switch ($this->type)
		{
			case Param_type::bool:
			case Param_type::int:
				return is_int($value) ? $value : (int) $value;
				break;
			case Param_type::float:
				return is_float($value) ? $value : (float) $value;
				break;
			default:
				return "'" . sed_sql_prep($value) . "'";
				break;
		}
	}

	/**
	 * Imports parameter value from user input
	 * @param string $method Request method: P - post, G - get.
	 * @param string $prefix Input name prefix
	 * @return mixed Value on success or NULL if validation failed
	 */
	public function import($method = 'P', $prefix = '')
	{
		// TODO throw exceptions
		switch ($this->type)
		{
			case Param_type::bool:
				$value = sed_import($prefix . $this->name, $method, 'BOL');
				if ($value == '' && $this->required)
				{
					$value = NULL;
				}
				else
				{
					$value = (bool) $value;
				}
				break;
			case Param_type::float:
				$value = sed_import($prefix . $this->name, $method, 'TXT');
				if (strstr($value, ',') && strstr($value, '.'))
				{
					$value = str_replace(',', '', $value);
				}
				if (preg_match('#^\d+$#', $value) || preg_match('#\d+\.\d+#', $value))
				{
					$value = (float) $value;
				}
				else
				{
					$value = NULL;
				}
				break;
			case Param_type::int:
				$value = sed_import($prefix . $this->name, $method, 'INT');
				if (!is_numeric($value) && $this->required)
				{
					$value = NULL;
				}
				else
				{
					$value = (int) $value;
				}
				break;
			default:
				$value = $this->allow_html ? sed_import($prefix . $this->name, $method, 'HTM')
					: sed_import($prefix . $this->name, $method, 'TXT');
				if (empty($value) && $this->required)
				{
					$value = NULL;
				}
				break;
		}
		if (empty($value) && !is_null($this->default))
		{
			$value = $this->default;
		}
		return $value;
	}

	/**
	 * Renders param value display in a correct way
	 * @param mixed $value
	 * @return string Rendered display element
	 * @see Param::display_input(), Param::title, Param::hint
	 */
	public function display($value)
	{
		switch ($this->type)
		{
			case Param_type::bool:
				$checked = (is_null($value) || !$value) ? '' : ' checked="checked"';
				$disp = '<input type="checkbox" readonly="readonly" name="' . $prefix . $this->name . '"' . $checked . $class . ' />';
				break;
			default:
				$disp = sed_cc($value);
		}
		return $disp;
	}

	/**
	 * Renders param input display in a correct way
	 * @param mixed $value
	 * @param string $prefix Input name prefix
	 * @return string
	 * @see Param::display(), Param::title, Param::hint
	 */
	public function display_input($value = NULL, $prefix = '')
	{
		if (is_null($value) && !is_null($this->default))
		{
			$value = $this->default;
		}
		if ($this->display)
		{
			$class = empty($this->input_class) ? '' : ' class="' . $this->input_class . '"';
			switch ($this->type)
			{
				case Param_type::bool:
					$checked = (is_null($value) || !$value) ? '' : ' checked="checked"';
					$disp = '<input type="checkbox" name="' . $prefix . $this->name . '"' . $checked . $class . ' />';
					break;
				default:
					$value = is_null($value) ? '' : ' value="' . sed_cc($value) . '"';
					$disp = '<input type="text" name="' . $prefix . $this->name . '"' . $value . $class . ' />';
			}
			// Rest types are same or must be overloaded
		}
		else
		{
			$disp = '<input type="hidden" name="' . $prefix . $this->name . '" value="'.sed_cc($value).'" />';
		}
		return $disp;
	}
}
?>
