<?php
/**
 * @package CCL
 * @version 0.0.1
 * @author trustmaster
 * @copyright (c) 2009 Vladimir Sibirov
 * @license BSD
 */

/**
 * Controller Unit. Controller adds logic to Entity and connects it with View
 */
class Unit extends Entity
{
	/**
	 * @var array Relations (relation class names) this Unit participates in
	 */
	public $relations = array();
	/**
	 * @var array Entities which reference to this entity
	 */
	public $references = array();
	/**
	 * @var string Zone name used in URLs
	 */
	public $zone;
	/**
	 * @var string Unit title (i18n-d)
	 */
	public $title;
	/**
	 * @var string Name of the Item class representing objects for this Unit
	 */
	public $item_class = 'Item';
	/**
	 * @var string Item title (i18n-d)
	 */
	public $item_title = '';
	/**
	 * @var string Sorting order
	 */
	public $order = '';

	/**
	 * Initializes controller Unit
	 * @param string $name Unit name
	 * @param string $item_class Name of the Item class representing objects for this Unit
	 * @param string $prefix Table prefix
	 * @param string $col_prefix Column prefix
	 * @param array $params Array of Param objects
	 * @param string $primary_key Primary key
	 */
	public function __construct($name, $item_class = 'Item', $prefix = '', $col_prefix = '',
		$params = NULL, $primary_key = 'id')
	{
		global $L;
		parent::__construct($name, $prefix, $col_prefix, $params, $primary_key);
		$this->item_class = $item_class;
		$this->title = empty($L[$name]) ? $name : $L[$name];
		$this->item_title = empty($L[$item_class]) ? $item_class : $L[$item_class];
		$this->zone = mb_strtolower($this->name);
	}

	/**
	 * Renders add/edit form display. A form will be empty if object has no data and if some data
	 * was imported, it will be already in the inputs.
	 * TPL designer must specify unit parameters explicitly.
	 * TPL example:
	 * <!-- BEGIN: MY_UNIT -->
	 * <form action="{ACTION}" method="post">
	 * <stong>{FOO_TITLE}:</strong> {FOO_INPUT} <em>{FOO_HINT}</em>
	 * <stong>{BAR_TITLE}:</strong> {BAR_INPUT} <em>{BAR_HINT}</em>
	 * <input type="submit" value="Submit" />
	 * </form>
	 * <!-- END: MY_UNIT -->
	 *
	 * @param XTemplate $tpl Layout template object
	 * @param string $block Full block name to render the object in, e.g. 'MAIN.MY_UNIT'
	 * @param string $action Action that is being performed by the form. Usually 'edit', 'add' or 'remove'
	 * @param Item $item An imported item if performing edit rather than add
	 * @see Item::import(), Unit::display_form_auto()
	 */
	public function display_form($tpl, $block = 'MAIN', $action = 'update', $item = NULL)
	{
		global $z, $m, $a, $pl;
		$act_url = defined('SED_MODULE') ? sed_url(SED_MODULE, "z=$z&m=$m&a=$action") : sed_url('plug', "e={$pl['pl_code']}&z=$z&m=$m&a=$action");
		$tpl->assign('ACTION', $act_url);
		foreach ($this->data as $name => $param)
		{
			$value = !is_null($item) && isset($item->{$name}) ? $item->{$name} : NULL;
			if ($param->display)
			{
				if (!is_null($item) && $param instanceof Reference  && $param->entity == $this)
				{
					$param_input = $param->display_input($value, $this->col_prefix, $item->id);
				}
				else
				{
					$param_input = $param->display_input($value, $this->col_prefix);
				}
				$tpl->assign(array(
					mb_strtoupper($name) . '_TITLE' => sed_cc($param->title),
					mb_strtoupper($name) . '_HINT' => sed_cc($param->hint),
					mb_strtoupper($name) . '_INPUT' => $param_input
				));
			}
			elseif(!is_null($value))
			{
				$tpl->assign(mb_strtoupper($name) . '_INPUT',
					$param->display_input($value, $this->col_prefix));
			}
		}
		$tpl->parse($block);
	}

	/**
	 * Row-based automatic form render
	 * TPL designer specifies parameters implicitly with row block.
	 * TPL example:
	 * <!-- BEGIN: MY_UNIT -->
	 * <!-- BEGIN: MY_PARAM -->
	 * <stong>{PARAM_TITLE}:</strong> {PARAM}
	 * <!-- END: MY_PARAM -->
	 * <!-- END: MY_UNIT -->
	 *
	 * @param XTemplate $tpl Layout template object
	 * @param string $block Full block name to render the object in, e.g. 'MAIN.MY_UNIT'
	 * @param string $row_block Row block name for parameter row, e.g. 'MY_PARAM'
	 * @param string $action Action that is being performed by the form. Usually 'edit', 'add' or 'remove'
	 * @param Item $item An imported item if performing edit rather than add
	 * @param string $title_tag Tag name for parameter title
	 * @param string $input_tag Tag name for parameter value input
	 * @param string $hint_tag Tag name for hint displayed for the input
	 * @see Unit::display_form()
	 */
	public function display_form_auto($tpl, $block, $row_block, $action = 'update', $item = NULL,
		$title_tag = 'PARAM_TITLE', $input_tag = 'PARAM_INPUT', $hint_tag = 'PARAM_HINT')
	{
		global $z, $m, $a, $pl;
		$act_url = defined('SED_MODULE') ? sed_url(SED_MODULE, "z=$z&m=$m&a=$action") : sed_url('plug', "e={$pl['pl_code']}&z=$z&m=$m&a=$action");
		$tpl->assign('ACTION', $act_url);
		foreach ($this->data as $name => $param)
		{
			$value = !is_null($item) && isset($item->{$name}) ? $item->{$name} : NULL;
			if ($param->display)
			{
				if (!is_null($item) && $param instanceof Reference  && $param->entity == $this)
				{
					$param_input = $param->display_input($value, $this->col_prefix, $this->id);
				}
				else
				{
					$param_input = $param->display_input($value, $this->col_prefix);
				}
				$tpl->assign(array(
					$title_tag => sed_cc($param->title),
					$hint_tag => sed_cc($param->hint),
					$input_tag => $param_input
				));
				$tpl->parse($block . '.' . $row_block);
			}
		}
		$tpl->parse($block);
	}

	/**
	 * Renders a simple list of objects to be displayed on screen.
	 *
	 * @param XTemplate $tpl Layout template object
	 * @param string $block Full block name to render the object in, e.g. 'MAIN.MY_UNIT'
	 * @param string $filter Filter condition
	 * @param int $start Index of an item in the result set to start with
	 * @param int $items Max. number of items to select from result set. NULL - unlimited.
	 */
	public function display_list($tpl, $block = 'MAIN', $filter = '', $start = 0, $items = NULL)
	{
		$limit = (!is_null($items)) ? "LIMIT $start, $items" : '';
		$filter = empty($filter) ? '' : 'WHERE ' . $filter;
		$order = empty($this->order) ? '' : 'ORDER BY ' . $this->order;
		$res = sed_sql_query("SELECT * FROM `" . $this->table_name . "` $filter $order $limit");
		while ($row = sed_sql_fetchassoc($res))
		{
			$item = new $this->item_class($this, $row);
			$item->display($tpl, $block . '.ITEM');
		}
		sed_sql_freeresult($res);
	}

	/**
	 * Renders an editable list of objects to edit many instances of the entity at once
	 *
	 * @param XTemplate $tpl Layout template object
	 * @param string $block Full block name to render the object in, e.g. 'MAIN.MY_UNIT'
	 * @param string $filter Filter condition
	 * @param int $start Index of an item in the result set to start with
	 * @param int $items Max. number of items to select from result set. NULL - unlimited.
	 */
	public function display_list_edit($tpl, $block = 'MAIN', $filter = '', $start = 0, $items = NULL)
	{
		// TODO
	}

	/**
	 * Basic add scenario
	 */
	public function add()
	{
		global $cfg, $a, $z, $t, $pl;
		$item = new $this->item_class($this);
		if ($a == 'add')
		{
			if($item->import('P', $this->col_prefix))
			{
				$id = $item->save();
				$loc_url = defined('SED_MODULE') ? sed_url(SED_MODULE, "z=$z", '', TRUE) : sed_url('plug', "e={$pl['pl_code']}&z=$z", '', TRUE);
				header('Location: ' . $loc_url);
				exit;
			}
			else
			{
				$error_string = 'Не все поля заполнены верно'; // FIXME i18n
			}
		}

		$t = new XTemplate(mod_skinfile('form', $this->name));
		// Check a filter
		$filter = '';
		foreach ($this->data as $name => $param)
		{
			if ($param instanceof Reference)
			{
				$filter_id = (int) sed_import($name, 'G', 'INT');
				if ($filter_id > 0)
				{
					$filter = defined('SED_MODULE') ?
						sed_url(SED_MODULE, 'z=' . $param->entity->zone . '&m=display&id=' . $filter_id)
						: sed_url('plug', 'e=' . $pl['pl_code'] . '&z=' . $param->entity->zone . '&m=display&id=' . $filter_id);
					break;
				}
			}
		}
		$back_url = defined('SED_MODULE') ? sed_url(SED_MODULE, "z=$z") : sed_url('plug', "e={$pl['pl_code']}&z=$z");
		$t->assign(array(
			'TITLE' => 'Добавляется ' . $this->item_title, // FIXME i18n
			'BACK_URL' => empty($filter) ? $back_url : $filter
		));
		if (!empty($error_string))
		{
			$t->assign('ERROR_MESSAGE', $error_string);
			$t->parse('MAIN.ERROR');
		}
		$this->display_form_auto($t, 'MAIN', 'PARAM', 'add', $item);
	}

	/**
	 * Basic edit scenario
	 */
	public function edit()
	{
		global $cfg, $id, $id_str, $z, $m, $a, $t, $pl;
		if ($id > 0 || is_array($id))
		{
			$item = new $this->item_class($this, $id);
			if ($a == 'update')
			{
				if ($item->import('P', $this->col_prefix))
				{
					if ($item->save())
					{
						$error_string = 'Изменения сохранены успешно'; // FIXME i18n
					}
					else
					{
						$error_string = 'Поля не изменились или произошла ошибка при сохранении';
					}
				}
				else
				{
					$error_string = 'Не все поля заполнены верно'; // FIXME
				}
			}
			$t = new XTemplate(mod_skinfile('form', $this->name));
			// Check a filter
			$filter = '';
			foreach ($this->data as $name => $param)
			{
				if ($param instanceof Reference)
				{
					$filter_id = (int) sed_import($name, 'G', 'INT');
					if ($filter_id > 0)
					{
						$filter = defined('SED_MODULE') ?
						sed_url(SED_MODULE, 'z=' . $param->entity->zone . '&m=display&id=' . $filter_id)
						: sed_url('plug', 'e=' . $pl['pl_code'] . '&z=' . $param->entity->zone . '&m=display&id=' . $filter_id);
						break;
					}
				}
			}
			$back_url = defined('SED_MODULE') ? sed_url(SED_MODULE, "z=$z") : sed_url('plug', "e={$pl['pl_code']}&z=$z");
			$t->assign(array(
				'TITLE' => 'Редактируется ' . $this->item_title . ' ' . sed_cc($item->name), // FIXME i18n
				'BACK_URL' => empty($filter) ? $back_url : $filter
			));
			if (!empty($error_string))
			{
				$t->assign('ERROR_MESSAGE', $error_string);
				$t->parse('MAIN.ERROR');
			}
			$act_id = is_array($id) ? $id_str : '&id=' . $id;
			$this->display_form_auto($t, 'MAIN', 'PARAM', 'update' . $act_id, $item);
		}
		else
		{
			// error
			$loc_url = defined('SED_MODULE') ? sed_url(SED_MODULE, "z=$z", '', TRUE) :  sed_url('plug', "e={$pl['pl_code']}&z=$z", '', TRUE);
			header('Location: ' . $loc_url);
			exit;
		}
	}

	/**
	 * Basic remove scenario
	 */
	public function remove()
	{
		global $cfg, $id, $id_str, $z, $m, $a, $t, $pl;
		if ($id > 0 || is_array($id))
		{
			$item = new $this->item_class($this, $id);
			if ($a == 'remove')
			{
				$item->delete();
				$loc_url = defined('SED_MODULE') ? sed_url(SED_MODULE, "z=$z", '', TRUE) :  sed_url('plug', "e={$pl['pl_code']}&z=$z", '', TRUE);
				header('Location: ' . $loc_url);
				exit;
			}
			$t = new XTemplate(mod_skinfile('delete', $this->name));
			$act_id = is_array($id) ? $id_str : '&id=' . $id;
			$back_url = defined('SED_MODULE') ? sed_url(SED_MODULE, "z=$z") : sed_url('plug', "e={$pl['pl_code']}&z=$z") ;
			$confirm_url = defined('SED_MODULE') ? sed_url(SED_MODULE, "z=$z&m=$m&a=remove$act_id")
				: sed_url('plug', "e={$pl['pl_code']}&z=$z&m=$m&a=remove$act_id");
			$t->assign(array(
				'TITLE' => 'Удаляется ' . $this->title . ' ' . sed_cc($item->name), // FIXME i18n
				'BACK_URL' => $back_url,
				'CONFIRM_URL' => $confirm_url
			));
		}
		else
		{
			// error
			$loc_url = defined('SED_MODULE') ? sed_url(SED_MODULE, "z=$z", '', TRUE) :  sed_url('plug', "e={$pl['pl_code']}&z=$z", '', TRUE);
			header('Location: ' . $loc_url);
			exit;
		}
	}

	/**
	 * Basic item listing scenario
	 */
	public function list_items()
	{
		global $cfg, $z, $m, $a, $t, $pl;
		$t = new XTemplate(mod_skinfile('list', $this->name, $this instanceof Relation));
		$t->assign(array(
			'TITLE' => $this->title,
			'ADD_URL' => defined('SED_MODULE') ? sed_url(SED_MODULE, "z=$z&m=add") : sed_url('plug', "e={$pl['pl_code']}&z=$z&m=add"),
			'BACK_URL' => defined('SED_MODULE') ? sed_url(SED_MODULE) : sed_url('plug', 'e=' . $pl['pl_code'])
		));
		// Apply filters by reference
		$filter = '';
		foreach ($this->data as $name => $param)
		{
			if ($param instanceof Reference)
			{
				$filter_id = (int) sed_import($name, 'G', 'INT');
				if ($filter_id > 0)
				{
					if (!empty($filter))
					{
						$filter .= ' AND ';
					}
					else
					{
						$t->assign(array(
							'ADD_URL' => defined('SED_MODULE') ? sed_url(SED_MODULE, "z=$z&m=add&$name=$filter_id")
								: sed_url('plug', "e={$pl['pl_code']}&z=$z&m=add&$name=$filter_id"),
							'BACK_URL' => defined('SED_MODULE') ? sed_url(SED_MODULE, 'z=' . $param->entity->zone . '&m=display&id=' . $filter_id)
								: sed_url('plug', 'e=' . $pl['pl_code'] . '&z=' . $param->entity->zone . '&m=display&id=' . $filter_id)
						));
					}
					$filter .= "`{$this->col_prefix}$name` = $filter_id";
				}
			}
		}
		$this->display_list($t, 'MAIN', $filter);
	}

	/**
	 * Basic item display scenario
	 */
	public function display_item()
	{
		global $cfg, $z, $m, $a, $t, $id, $pl;
		$t = new XTemplate(mod_skinfile('display', $this->name));
		$item = new $this->item_class($this, $id);
		$name = empty($item->name) ? $item->id : $item->name;
		$back_url = defined('SED_MODULE') ? sed_url(SED_MODULE, "z=$z") : sed_url('plug', "e={$pl['pl_code']}&z=$z") ;
		$t->assign(array(
			'TITLE' => $this->title . ': ' . $name,
			'BACK_URL' => $back_url
		));
		$item->display_auto($t, 'MAIN', 'PARAM');
	}

	/**
	 * Basic main unit scenario
	 */
	public function main()
	{
		global $m;
		if ($m == 'add')
		{
			$this->add();
		}
		elseif ($m == 'edit')
		{
			$this->edit();
		}
		elseif ($m == 'remove')
		{
			$this->remove();
		}
		elseif ($m == 'display')
		{
			$this->display_item();
		}
		else
		{
			$this->list_items();
		}
	}
}
?>
